function PTB_Protocol_Gen_ver3(subjectNum, gainsloss, tr, trialduration, DiscAcq, ParametricMod, path_in, path_out, PRT)
%PTB_PROTOCOL_GEN Generates PRT Files from Psychtoolbox Data Files
%
%INPUTS:
%       subjectNum - Specifies subject file to be loaded
%       gainsloss - A string of either 'gains' or 'loss' to indicate the domain to extract
%       tr - Temporal resolution of the visual data, in seconds
%       trialduration - Number of seconds to analyze after trial onset
%       DiscAcq - Number of seconds to discard at the beginning of each block
%       ParametricMod - which value to use as parameter? Values: 'RewardValue', 'RiskLevel', 'AmbiguityLevel', 'SV', or ''
%       path_in - the dominant folder that both auxiliary function files and data files are stored in
%       path_out - the folder to save the generated PRT files into
%       PRT - a struct of settings for the PRT file
%
%OUTPUT: PRT files for given domain in the folder specified by `path_out`
%
% NOTE: In order to create a protocol file with onsets measured in seconds,
% simply switch tr value to 1

%% Process arguments
% Store in logical value whether protocol should be generated for gains or losses
is_gains = strcmp(gainsloss, 'gains');
if (~is_gains && ~strcmp(gainsloss, 'loss'))
  error('Aborting: `gainsloss` must be set to either "gains" or "loss"!')
  return
end

% Set NumParametricWeights on the basis of ParametricMod
permissible_parameters = {'RewardValue', 'RiskLevel', 'AmbiguityLevel', 'SV'};
if sum(strcmp(ParametricMod, permissible_parameters)) 
  % using sum because strcmp returns a matrix of logicals if string is compared 
  % with an array of strings
  NumParametricWeights = 1;
elseif strcmp(ParametricMod, '')
  NumParametricWeights = 0;
else
  error('Aborting: `ParametricMod` must be set to a valid parameter value!')
  return
end
PRT.ParametricWeights = num2str(NumParametricWeights);

% For PRT properties, fix colors into correct order
if ~is_gains
  Colors = {PRT.ColorAmb_Loss, PRT.ColorRisk_Loss};
  alt_colors = {PRT.ColorAmb_Gains, PRT.ColorRisk_Gains};
else
  Colors = {PRT.ColorAmb_Gains, PRT.ColorRisk_Gains};
  alt_colors = {PRT.ColorAmb_Loss, PRT.ColorRisk_Loss};
end

%% Load Gains and Loss Data files
% Add the directory & all subdirs to path
addpath(genpath(path_in)); 

load(['RA_GAINS_' num2str(subjectNum) '_fitpar_constrained.mat']);
gdata = Data;
load(['RA_LOSS_' num2str(subjectNum) '_fitpar_constrained.mat']);
ldata = Data;

clearvars Data; % avoid accidental name collision

% Pick the domain to analyze
if (is_gains)
  data = gdata;
else
  data = ldata;
end

%% Get correct block order
block_order = getBlockOrder(is_gains, gdata, ldata);

%% Compute subjective value of each choice
% NOTE: Where does the alpha and beta come from?
for reps = 1:length(data.choice)
  sv(reps, 1) = ambig_utility(0, ...
      data.vals(reps), ...
      data.probs(reps), ...
      data.ambigs(reps), ...
      data.alpha, ...
      data.beta, ...
      'ambigNrisk');
end

% Flip sign, since the data files store only value magnitudes 
if ~is_gains
  sv(:, 1) = -1 * sv(:, 1);
end

%% Load onset times
gon = PTB_Protocol_OnsetExtract(gdata);
lon = PTB_Protocol_OnsetExtract(ldata);

% Extract per-block time info from returned argument
gonsets = {gon.b1, gon.b2, gon.b3, gon.b4};
lonsets = {lon.b1, lon.b2, lon.b3, lon.b4};

%% Iterate over blocks in domain
% NOTE: 4 is magic number, since currently each domain has exactly 4 blocks
for blocknum = 1:4
  %% Select onset/offset time block to use
  if is_gains
    prtblock = gonsets{blocknum};
  else
    prtblock = lonsets{blocknum};
  end

  %% Process onset/offset times
  [ onsets, offsets ] = getBlockOnsetOffset(prtblock, DiscAcq, tr, trialduration);

  %% Compute values from current block
  % Get indices for risk trials and ambiguity trials (30 values, regularly spaced)
  current_block_range = (2:31) + (blocknum - 1) * 31;

  % Divide blocks by type of lottery (store indices with risk-only vs. risk-and-ambiguity)
  amb_index = data.ambigs(current_block_range) > 0;
  risk_index = data.ambigs(current_block_range) == 0;

  % Parametric weights for current block
  if NumParametricWeights > 0
    block_amt = data.vals(current_block_range);
    block_rlevel = data.probs(current_block_range);
    block_alevel = data.ambigs(current_block_range);
    block_sv = sv(current_block_range);

    if ~is_gains
      block_amt = -1 * block_amt;
    end
  end

  %% Select what parametric value to write (or not write) into PRT file
  % Store the basic onset/offset, computed earlier
  block_amb = [onsets(amb_index) offsets(amb_index)];
  block_risk = [onsets(risk_index) offsets(risk_index)];

  % Add the selected parametric value if required
  if NumParametricWeights > 0
    if strcmp(ParametricMod, 'RewardValue')
      block_amb = [block_amb block_amt(amb_index)];
      block_risk = [block_risk block_amt(risk_index)];
    elseif strcmp(ParametricMod, 'RiskLevel')
      block_amb = [block_amb block_rlevel(amb_index)];
      block_risk = [block_risk block_rlevel(risk_index)];
    elseif strcmp(ParametricMod, 'AmbiguityLevel')
      block_amb = [block_amb block_alevel(amb_index)];
      block_risk = [block_risk block_alevel(risk_index)];
    elseif strcmp(ParametricMod, 'SV')
      block_amb = [block_amb block_sv(amb_index)];
      block_risk = [block_risk block_sv(risk_index)];
    end
  end

  %% Write file to txt
  % TODO: Called once per block - abstract into a subfunction / nested function?

  % Determine session for filename
  if blocknum <= 2
    Session = 'S1';
  else
    Session = 'S2';
  end

  % Open file for writing
  if strcmp(ParametricMod, '')
    ParametricMod = 'none';
  end
  fname = [path_out num2str(subjectNum) '_' Session '_block' block_order{blocknum} ...
      '_' gainsloss num2str(blocknum) '_type_' ParametricMod '.prt'];
  fileID = fopen(fname, 'w');

  % TODO: Figure out how %Ns works -- it's highly unlikely that both values
  % actually do a useful thing
  fprintf(fileID, '%12s %10s\r\n \r\n', 'FileVersion:', PRT.FileVersion);
  fprintf(fileID, '%17s %11s\r\n \r\n', 'ResolutionOfTime:', PRT.ResolutionOfTime);
  fprintf(fileID, '%11s %21s\r\n \r\n', 'Experiment:', PRT.Experiment);
  fprintf(fileID, '%16s %16s\r\n', 'BackgroundColor:', PRT.BackgroundColor);
  fprintf(fileID, '%10s %16s\r\n', 'TextColor:', PRT.TextColor);
  fprintf(fileID, '%16s %10s\r\n', 'TimeCourseColor:', PRT.TimeCourseColor);
  fprintf(fileID, '%16s %6s\r\n', 'TimeCourseThick:', PRT.TimeCourseThick);
  fprintf(fileID, '%19s %11s\r\n', 'ReferenceFuncColor:', PRT.ReferenceFuncColor);
  fprintf(fileID, '%19s %3s\r\n \r\n', 'ReferenceFuncThick:', PRT.ReferenceFuncThick);

  % TODO: Shouldn't ParametricWeights be always included, even if they *are* 0?
  if NumParametricWeights > 0
      fprintf(fileID, '%18s %4s\r\n', 'ParametricWeights:', PRT.ParametricWeights);
  end

  fprintf(fileID, '\r\n%15s %7s\r\n \r\n', 'NrOfConditions:', PRT.NrOfConditions);

  % NOTE: Empty conditions -- set here for purposes of consistent order. 
  % Equivalent but opposite block further below
  if ~is_gains & NumParametricWeights ~= 0
    fprintf(fileID, '%9s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_gains', '0', 'Color:', alt_colors{1});
    fprintf(fileID, '%14s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_gains x p1', '0', 'Color:', '0 0 0');
    fprintf(fileID, '%10s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_gains', '0', 'Color:', alt_colors{2});
    fprintf(fileID, '%15s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_gains x p1', '0', 'Color:', '0 0 0');
  elseif ~is_gains & NumParametricWeights == 0
    fprintf(fileID, '%9s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_gains', '0', 'Color:', alt_colors{1});
    fprintf(fileID, '%10s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_gains', '0', 'Color:', alt_colors{2});
  end

  % Print the ambiguity block for given domain
  fprintf(fileID, '%7s\r\n', ['Amb_' gainsloss]);
  fprintf(fileID, '%4s\r\n', num2str(length(block_amb)));

  if NumParametricWeights == 0
      fprintf(fileID, '%4.0f\t %3.0f\r\n', block_amb');
  else
      fprintf(fileID, '%4.0f\t %3.0f\t %1.3f\r\n', block_amb');
  end

  fprintf(fileID, '%6s %7s\r\n', 'Color:', Colors{1});
  

  % Print the risk block for given domain
  fprintf(fileID, '\r\n%9s\r\n', ['Risk_' gainsloss]);
  fprintf(fileID, '%4s\r\n', num2str(length(block_risk)));

  if NumParametricWeights == 0
    fprintf(fileID, '%4.0f\t %3.0f\r\n', block_risk');
  else
    fprintf(fileID, '%4.0f\t %3.0f\t %1.3f\r\n', block_risk');
  end

  fprintf(fileID, '%6s %7s\r\n', 'Color:', Colors{2});
  
  % Equivalent empty-condition block from above
  if is_gains & NumParametricWeights ~= 0
    fprintf(fileID, '\r\n%8s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_loss', '0', 'Color:', alt_colors{1});
    fprintf(fileID, '\r\n%13s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_loss x p1', '0', 'Color:', '0 0 0');
    fprintf(fileID, '%9s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_loss', '0', 'Color:', alt_colors{2});
    fprintf(fileID, '%14s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_loss x p1', '0', 'Color:', '0 0 0');
  elseif is_gains & NumParametricWeights == 0
    fprintf(fileID, '\r\n%8s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Amb_loss', '0', 'Color:', alt_colors{1});
    fprintf(fileID, '%9s\r\n%3s\r\n%4s %6s\r\n\r\n', 'Risk_loss', '0', 'Color:', alt_colors{2});
  end

  fclose(fileID);
end
end
